

package org.dronus.graph.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.PopupMenu;

import org.dronus.graph.Node;
import org.dronus.graph.PopupNode;

public class LayoutableNode extends Node implements PopupNode{

	public static Color
		FIXED_COLOR        = Color.RED,
		FIXED_SELECT_COLOR = new Color(255,150,0),
		SELECT_COLOR       = new Color(208, 200, 0),   
		TEXT_COLOR              = Color.WHITE,
		TAG_COLOR              = new Color(0,200,0);
	public static final Font
		TEXT_FONT = new Font("LucidaBright",Font.PLAIN,10),
		TAG_FONT = new Font("LucidaBright",Font.PLAIN,8);    	
	public static Color
		DEFAULT_COLOR      = Color.DARK_GRAY,	
		DEFAULT_COLOR_AA   = new Color(127,127,127,100);	
	static Color defaultColor=DEFAULT_COLOR; //default no AA selected
	static public void setAntialias(boolean aa){defaultColor = (aa ? DEFAULT_COLOR_AA : DEFAULT_COLOR  );}
	
	NodeRenderer nodeRenderer=new NodeRenderer(this);
	
	public double drawx,drawy;
	protected double x,y,dx,dy;
	public void stop(){dx=dy=0;};
		
    public boolean fixed = false, justMadeLocal = false, markedForRemoval = false;    
     
    public int visibleEdgeCnt;

	protected String id,type,lbl;
	
	static final String
    	NT_TYPE="Typ", 
    	NT_PUNKT="Punkt",
		NT_FILE="Datei";
	
	protected LayoutableNode( String id, String _lbl, String _type )   {
        this.id=id;
		if ( _lbl == null ) lbl = id;
        else lbl = _lbl;
        setType(_type);
		//x = Math.random()*200-100;	y = Math.random()*200-100;
		if (id!=null) {
			float i=(new Integer(id)).floatValue()/400.0f;		
			x = Math.cos(i)*200;	y = Math.sin(i)*200;
		}        
    }
    
    public void setID( String id ) { this.id = id;}
    public String getID() { return id; }
    public void setLocation( Point p ) { this.x = p.x;  this.y = p.y;}
    public Point getLocation() { return new Point((int)x,(int)y); }
	public void setType( String _type ) {
        type = _type;
        if(type.equals(NT_PUNKT))  	nodeRenderer=new PunktNodeRenderer(this);
		else if(type.equals(NT_FILE))  	nodeRenderer=new FileNodeRenderer(this);
		else if(type.equals(NT_TYPE))nodeRenderer=new NodeRenderer(this);
		else nodeRenderer=new NodeRenderer(this);		
    }
    public String getType() {   return type; }
    public void setLabel( String label ) {   lbl = label; nodeRenderer.update();}
    public String getLabel() { return lbl; }
    public int visibleEdgeCount() {   return visibleEdgeCnt; }
    public boolean intersects( Dimension d ) {
        return ( drawx > 0 && drawx < d.width && drawy>0 && drawy < d.height );
    }

    public boolean containsPoint( double px, double py ) {
		return nodeRenderer.containsPoint(px,py);
    }

    public void paint( Graphics g, boolean selected, boolean print ) {
        nodeRenderer.paint(g,(int)drawx, (int)drawy, selected,print);
    }
	public void draw( Graphics g, int x, int y, boolean selected, boolean print ) {
		nodeRenderer.paint(g,x,y,selected,print);
	}

    protected Color getPaintBackColor(boolean selected) {
       return nodeRenderer.getPaintBackColor(selected);
    }        
	
	public void extendNodePopup(PopupMenu m, MutaGraphPanel tgp) {
		if(nodeRenderer instanceof PopupNode) ((PopupNode)nodeRenderer).extendNodePopup(m,tgp); 
	}

	public void defaultAction() {
		if(nodeRenderer instanceof ActionNode) ((ActionNode)nodeRenderer).defaultAction();
	}

}
