package org.dronus.graph.gui;

import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.image.BufferedImage;

import org.dronus.dope.FilePool;
import org.dronus.exploder.ImageLoader;


public class NodeRenderer implements ActionNode{
		
	protected LayoutableNode n;	
	protected int w,h,tw;
	private Image img;
    
	protected NodeRenderer(LayoutableNode _n){
		n=_n;
		update();
	}

	static FontMetrics makeFontMetrics (){
		return (new BufferedImage(100,100,BufferedImage.TYPE_INT_RGB)).getGraphics().getFontMetrics(LayoutableNode.TEXT_FONT);
	}
	protected static FontMetrics fontMetrics = makeFontMetrics();
 
	public int getWidth() {return w; }
	public int getHeight() {return h; }
	
	public void paint( Graphics g, int ix, int iy, boolean selected, boolean print ) {
		   paintNodeBody(g,ix,iy,selected, print);
		   if ( n.visibleEdgeCount()<n.edgeCount() && !print ) {
			   int h = getHeight();
			   int w = getWidth();
			   int tagX = ix+(w-7)/2-2+w%2;
			   int tagY = iy-h/2-2;
			   char character;
			   int hiddenEdgeCount = n.edgeCount()-n.visibleEdgeCount();
			   character = (hiddenEdgeCount<9) ? (char) ('0' + hiddenEdgeCount) : '*';
			   paintSmallTag(g, tagX, tagY, character);
		   }
	   }


	public void paintNodeBody( Graphics g, int ix, int iy, boolean selected, boolean print) {
		  g.setFont(LayoutableNode.TEXT_FONT);
		  //int r = h/2+1; // arc radius
		  Color backCol = getPaintBackColor(selected);
		  g.setColor(backCol);
	
		  if (img !=null){
			  if (print)	g.drawImage(img, ix-16,iy-42,null);
			  g.fillOval(ix - w/2, iy - h / 2, w, h );
			  g.setColor(LayoutableNode.TEXT_COLOR);        
			  if (!print) g.drawImage(img, ix,iy ,null);        	        	  		
			  g.drawString(n.getLabel(), ix - (w-12)/2, iy + fontMetrics.getDescent() +1);
		  } else {
			  g.fillOval(ix - tw/2, iy - h / 2, tw, h );
			  g.setColor(LayoutableNode.TEXT_COLOR);
			  g.drawString(n.getLabel()+" ("+n.getType()+")",ix- (tw-12)/2 ,iy + fontMetrics.getDescent() +1);
		  }
		
	  }
	
	/** Paints a tag with containing a character in a small font. */
	public void paintSmallTag(Graphics g, int tagX, int tagY, 
							char character) {
		g.setColor(LayoutableNode.TAG_COLOR);
		g.fillRect(tagX, tagY, 8, 8);
		g.setColor(Color.BLACK);
		g.setFont(LayoutableNode.TAG_FONT);
		g.drawString(""+character, tagX+2, tagY+7);
	}
	
	protected Color getPaintBackColor(boolean selected) {
	   if ( selected ) {
		   if (!n.fixed) return LayoutableNode.SELECT_COLOR;
		   else return LayoutableNode.FIXED_SELECT_COLOR;
	   } else {
		   if (n.fixed) return LayoutableNode.FIXED_COLOR;
		   if (n.markedForRemoval) return new Color(100,60,40);
		   if (n.justMadeLocal) return new Color(255,220,200);
		   return LayoutableNode.defaultColor;            
	   }
   }      
	
	public void update(){
		if ( n.getLabel() != null ) {
			w=fontMetrics.stringWidth(n.getLabel())+12;
			h=fontMetrics.getHeight() + 6;
			tw=fontMetrics.stringWidth(n.getLabel()+"("+n.getType()+")")+12;            
		} else {tw=w=10;h=6;}
		img=ImageLoader.getImg(n.getType());		
	}
	public boolean containsPoint( double px, double py ) {
		return (( px > n.drawx-getWidth()/2) && ( px < n.drawx+getWidth()/2) 
				&& ( py > n.drawy-getHeight()/2) && ( py < n.drawy+getHeight()/2));
	}
	public void defaultAction(){
		FilePool.getDefaultPool().invokeBySomething(n.getLabel(),n.getType());
	}
}
